

#include "qhybase.h"
#include "config.h"
#ifndef __IMG132EDEF_H__
#define __IMG132EDEF_H__

#define GUIDE_EAST             0x00010010     /* 0 RA+ */
#define GUIDE_NORTH            0x00020020     /* 1 DEC+ */
#define GUIDE_SOUTH            0x00020040     /* 2 DEC- */
#define GUIDE_WEST             0x00010080     /* 3 RA- */

/**
 * @brief IMG132E class define
 *
 * include all functions for img132e
 */
class IMG132E:public QHYBASE
{
public:
  IMG132E();
  ~IMG132E();
  /**
   @fn uint32_t ConnectCamera(qhyccd_device *d,qhyccd_handle **h)
   @brief connect to the connected camera
   @param d camera deivce
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t ConnectCamera(qhyccd_device *d,qhyccd_handle **h);

  /**
   @fn uint32_t DisConnectCamera(qhyccd_handle *h)
   @brief disconnect to the connected camera
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t DisConnectCamera(qhyccd_handle *h);

  /**
   @fn uint32_t InitChipRegs(qhyccd_handle *h)
   @brief Init the registers and some other things
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t InitChipRegs(qhyccd_handle *h);

  /**
   @fn uint32_t IsChipHasFunction(CONTROL_ID id)
   @brief check the camera has the function or not
   @param id function id
   @return
   HAVE return QHYCCD_HAVE \n
   NOT HAVE return QHYCCD_NOTHAVE
   */
  uint32_t IsChipHasFunction(CONTROL_ID id);

  /**
      @fn virtual uint32_t GetChipMemoryLength()
      @brief get the image cost memory length
      @return
      success return memory length \n
      another QHYCCD_ERROR code on other failures
      */
  uint32_t GetChipMemoryLength();

  uint32_t SetChipOffset(qhyccd_handle *h,double offset);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  uint32_t SetChipWBRed(qhyccd_handle *h,double red);
  uint32_t SetChipWBGreen(qhyccd_handle *h,double red);
  uint32_t SetChipWBBlue(qhyccd_handle *h,double red);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  /**
   @fn uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i)
   @brief set the transfer speed to camera
   @param h camera control handle
   @param i speed level
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i);

  /**
   @fn uint32_t SetChipBinMode(qhyccd_handle *h,uint32_t wbin,uint32_t hbin)
   @brief set the camera offset
   @param h camera control handle
   @param wbin width bin
   @param hbin height bin
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBinMode(qhyccd_handle *h,uint32_t wbin,uint32_t hbin);

  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);

  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);

  /**
   @fn uint32_t BeginSingleExposure(qhyccd_handle *h)
   @brief begin single exposure
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t BeginSingleExposure(qhyccd_handle *h);

  /**
   @fn uint32_t CancelExposing(qhyccd_handle *handle)
   @param handle camera control handle
   @return
   on success,return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t CancelExposing(qhyccd_handle *handle);

  /**
   @fn uint32_t CancelExposingAndReadout(qhyccd_handle *h)
   @brief stop single exposure
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t CancelExposingAndReadout(qhyccd_handle *h);

  /**
   @fn uint32_t GetSingleFrame(qhyccd_handle *h,uint32_t *pW,uint32_t *pH,uint32_t * pBpp,uint32_t *pChannels,uint8_t *ImgData)
   @brief get single frame image data
   @param h camera control handle
   @param pW real width
   @param pH real height
   @param pBpp real depth bits
   @param pChannels real channels
   @param ImgData image data buffer
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetSingleFrame(qhyccd_handle *h,uint32_t *pW,uint32_t *pH,uint32_t * pBpp,uint32_t *pChannels,uint8_t *ImgData);

  /**
   @fn uint32_t BeginLiveExposure(qhyccd_handle *h)
   @brief begin live exposure
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t BeginLiveExposure(qhyccd_handle *h);

  /**
   @fn uint32_t StopLiveExposure(qhyccd_handle *h)
   @brief stop live exposure
   @param h camera control handle
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t StopLiveExposure(qhyccd_handle *h);


  void w_i2c_imx035(qhyccd_handle *h,unsigned char addr,unsigned char dat);

  uint32_t ReservedFunction(uint32_t width,uint32_t height,uint32_t bpp,uint32_t channels,uint8_t *ImgData);

  uint32_t ImgX;
  uint32_t ImgY;
  uint32_t ShowImgX;
  uint32_t ShowImgY;
  uint32_t ShowImgX_Start;
  uint32_t ShowImgY_Start;
};
#endif

