

#include "qhy5iii183base.h"
#include "config.h"
#ifndef QHY183_DEF
#define QHY183_DEF


/**
 * @brief QHY183C class define
 *
 * include all functions for qhy183c
 */
class QHY183:public QHY5III183BASE
{
public:
  QHY183();
  ~QHY183();

  /**
   @fn uint32_t IsChipHasFunction(CONTROL_ID id)
   @brief check the camera has the function or not
   @param id function id
   @return
   HAVE return QHYCCD_HAVE \n
   NOT HAVE return QHYCCD_NOTHAVE
   */
  uint32_t IsChipHasFunction(CONTROL_ID id);

  /**
      @fn virtual uint32_t GetChipMemoryLength()
      @brief get the image cost memory length
      @return
      success return memory length \n
      another QHYCCD_ERROR code on other failures
      */
  uint32_t GetChipMemoryLength();
  /* add some functions for qhy183m,
  because we change the ddr of 183m,so its functions need to
  differ from qhy183c
  */
  uint32_t BeginLiveExposure(qhyccd_handle *h);
  uint32_t StopLiveExposure(qhyccd_handle *h);
  uint32_t BeginSingleExposure(qhyccd_handle *h) ;
  uint32_t GetSingleFrame(qhyccd_handle *h, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);
  static void ThreadCountExposureTime(void *p);
  void ThreadCountExposureTimeStart(qhyccd_handle *h);
private:

  double pllratio,oldpllratio; //!< inter pll ratio

  uint16_t vwinpos,oldvwinpos,vwidcut,oldvwidcut;

  uint8_t oldcamddr;

  uint32_t hmax_ref,vmax_ref;
  uint32_t vmax,hmax,oldvmax,oldhmax;
  uint32_t framelength,oldframelength;
  uint32_t oldchipoutputsizex,oldchipoutputsizey,oldchipoutputbits;
  uint32_t patchvnumber,oldpatchvnumber;
  int32_t shr,svr,spl,oldshr,oldsvr,oldspl;
  double oldtotalcamred2green,totalcamred2green,oldtotalcamgreen,totalcamgreen,oldtotalcamblue2green,totalcamblue2green,oldcamgain;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;

  uint8_t oldreg08,reg08,oldreg01,reg01;
  uint8_t oldlockframe,lockframe;

  uint16_t chiproiy,oldchiproiy,chiproiysize,oldchiproiysize;

  uint32_t oldampv_min,ampv_min,oldampv_max,ampv_max;
  uint8_t oldampv_onff,ampv_onff;

  uint8_t forcestop;

  int ApproxDDR,ApproxDDR_Target;
};
#endif
