
#include "config.h"
#include "qhy5iiibase.h"
#include "qhy5iiicoolbase.h"
#include "qhy5iiiddrcoolbase.h"

#ifndef QHY294_DEF
#define QHY294_DEF


/**
 * @brief QHY163C class define
 *
 * include all functions for qhy163c
 */
class QHY294:public QHY5IIICOOLBASE
{
public:
  QHY294();
  ~QHY294();

  /**
   @fn uint32_t IsChipHasFunction(CONTROL_ID id)
   @brief check the camera has the function or not
   @param id function id
   @return
   HAVE return QHYCCD_HAVE \n
   NOT HAVE return QHYCCD_NOTHAVE
   */
  uint32_t IsChipHasFunction(CONTROL_ID id);

  /**
      @fn virtual uint32_t GetChipMemoryLength()
      @brief get the image cost memory length
      @return
      success return memory length \n
      another QHYCCD_ERROR code on other failures
     */
  uint32_t GetChipMemoryLength();
  uint32_t InitChipRegs(qhyccd_handle *h) ;
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);
  uint32_t SetChipOffset(qhyccd_handle *h,double offset);
  uint32_t SetChipGain(qhyccd_handle *h,double gain);
  uint32_t SetChipWBRed(qhyccd_handle *h,double red);
  uint32_t SetChipWBGreen(qhyccd_handle *h,double green);
  uint32_t SetChipWBBlue(qhyccd_handle *h,double blue);
  uint32_t BeginLiveExposure(qhyccd_handle *h);
  uint32_t StopLiveExposure(qhyccd_handle *h);
  uint32_t SetChipExposeTime(qhyccd_handle *h,double time);
  uint32_t SetChipResolution(qhyccd_handle *h,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);
  uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY);
  uint32_t ReSetParams2cam(qhyccd_handle *h);
  uint32_t BeginSingleExposure(qhyccd_handle *h);
  //QinXiaoXu  START  20181019
  uint32_t GetLiveFrame(qhyccd_handle *h, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);
  //QinXiaoXu  END  20181019
  uint32_t GetSingleFrame(qhyccd_handle *pDevHandle,uint32_t *pW,uint32_t *pH,uint32_t *pBpp,uint32_t *pChannels,uint8_t *pImgData);
  uint32_t SetStreamMode(qhyccd_handle *h, uint8_t mode);
  uint32_t SetDDR(qhyccd_handle *h, double value);
  unsigned int readDDRNum(qhyccd_handle *handle);

  unsigned int ApproxDDR;
  unsigned int ApproxDDR_Target;
  double lastExposeTime;

  //QinXiaoXu  START  20181019
  uint32_t cutleftsize,cutrightsize,cuttopsize,cutbuttomsize;
  int  unbinningxstart ;
  int  unbinningystart;
  int  unbinningxsize ;
  int  unbinningysize ;
  //QinXiaoXu  END  20181019

};
#endif
