

#include "qhy5iiibase.h"
#include "qhy5iiicoolbase.h"
#include "qhy5iiiddrcoolbase.h"
#ifndef QHY550_DEF
#define QHY550_DEF


/**
 * @brief QHY5IIIG400M class define
 *
 * include all functions for qhy5iiig400m
 */
class QHY550:public QHY5IIICOOLBASE
{
public:
  QHY550();
  ~QHY550();

  /**
   @fn uint32_t IsChipHasFunction(CONTROL_ID id)
   @brief check the camera has the function or not
   @param id function id
   @return
   HAVE return QHYCCD_HAVE \n
   NOT HAVE return QHYCCD_NOTHAVE
   */
  void WriteCMOS(qhyccd_handle *handle,uint16_t index,uint16_t val);
  uint32_t InitChipRegs(qhyccd_handle *h);
  uint32_t IsChipHasFunction(CONTROL_ID id);


  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  /*
   @fn uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i)
   @brief set the transfer speed to camera
   @param h camera control handle
   @param i speed level
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);

  uint32_t SetChipOffset(qhyccd_handle *h,double offset);
  //uint32_t SetDDR(qhyccd_handle *h, double value);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);
  void UpdateParameters(qhyccd_handle *pDevHandle);

  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);
  uint32_t SetDDR(qhyccd_handle *h, double value) ;

  uint32_t SetChipBinMode(qhyccd_handle *h, uint32_t wbin, uint32_t hbin);

  uint32_t SetChipUSBTraffic(qhyccd_handle *h, uint32_t i);
  uint32_t GetLiveFrame(qhyccd_handle *h,uint32_t *pW,uint32_t *pH,uint32_t * pBpp,uint32_t *pChannels,uint8_t *ImgData);

  /**
   @fn uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i)
   @brief set hblank
   @param h camera control handle
   @param i hblank value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  //uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i);
  void initcmos(qhyccd_handle *pDevHandle);
  uint32_t GetChipMemoryLength();
  //   uint32_t GetSingleFrame(qhyccd_handle *h, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);

  uint32_t BeginLiveExposure(qhyccd_handle *h);
  uint32_t StopLiveExposure(qhyccd_handle *h);

  uint32_t  BeginSingleExposure(qhyccd_handle *h);
  uint32_t  GetSingleFrame(qhyccd_handle *h,uint32_t *pW,uint32_t *pH,uint32_t *pBpp,uint32_t *pChannels,uint8_t *ImgData);


private:
  double oldpllratio,pllratio; //!< inter pll ratio
  uint32_t longtimeflag;

  int32_t m_UsbTransferred;

  uint16_t digital_gain, analog_gain;



  uint16_t vwinpos, oldvwinpos, vwidcut, oldvwidcut;

  uint8_t oldcamddr;

  uint32_t hmax_ref, vmax_ref;
  uint32_t vmax, hmax, oldvmax, oldhmax;
  uint32_t framelength, oldframelength;
  uint32_t oldchipoutputsizex, oldchipoutputsizey, oldchipoutputbits;
  uint32_t oldchipoutputx, oldchipoutputy;

  uint32_t patchvnumber, oldpatchvnumber;

  int32_t oldeleshutter, eleshutter;

  double oldtotalcamred2green, totalcamred2green, oldtotalcamgreen, totalcamgreen, oldtotalcamblue2green, totalcamblue2green, oldcamgain;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;

  uint8_t oldreg08, reg08, oldreg01, reg01;
  uint8_t oldlockframe, lockframe;

  uint16_t chiproiy, oldchiproiy, chiproiysize, oldchiproiysize;

  uint32_t oldampv_min, ampv_min, oldampv_max, ampv_max;
  uint8_t oldampv_onff, ampv_onff;
  uint32_t cutleftsize, cutrightsize, cuttopsize, cutbottomsize;


  int ApproxDDR;
  int ApproxDDR_Target;
};
#endif
