
#include "qhy5iiicoolbase.h"
#include "config.h"
#ifndef QHY5III128BASE_DEF
#define QHY5III128BASE_DEF

#define USB_ENDPOINT  0x81




/**
 * @brief QHY367M class define
 *
 * include all functions for qhy367m
 */
class QHY5III128BASE:public QHY5IIICOOLBASE
{
public:
  QHY5III128BASE();
  ~QHY5III128BASE();

  //unsigned int readDDRNum(qhyccd_handle *h);

  void CMOS_TakePhoto(qhyccd_handle *h);

  uint32_t InitChipRegs(qhyccd_handle *h);

  void SpiPath(qhyccd_handle *handle,uint8_t val);

  void CMOSReset(qhyccd_handle *handle);

  void FX3SPIMode(qhyccd_handle *handle);

  void FPGASPIMode(qhyccd_handle *handle);

  void SetFreqDiv(qhyccd_handle *handle,uint8_t val);

  void IMX193_TakePhoto(qhyccd_handle *handle);

  void SetFPGACrop(qhyccd_handle *handle,uint32_t x,uint32_t y);

  void SetFPGAOutputBits(qhyccd_handle *handle,uint32_t val);

  void WriteCMOSOffset(qhyccd_handle *handle,uint32_t val);

  void WriteCMOSGain(qhyccd_handle *handle,uint32_t val);

  void WriteFPGADigitalGain(qhyccd_handle *handle,uint32_t val);

  void WriteCMOSAnalogGainRed(qhyccd_handle *handle,uint32_t val);

  void WriteCMOSAnalogGainBlue(qhyccd_handle *handle,uint32_t val);

  void WriteCMOSAnalogGainGreen(qhyccd_handle *handle,uint32_t val);

  void AMPVControl(qhyccd_handle *handle,uint8_t val);

  void InitSPISleep(qhyccd_handle *handle);

  void WriteCMOS(qhyccd_handle *handle,uint16_t index,uint16_t val);

  void WriteCMOSSHS(qhyccd_handle *handle,uint32_t val);

  void SetCMOSCrop(qhyccd_handle *handle,uint32_t ystart,uint32_t ysize);

  void SetSleepStart(qhyccd_handle *handle,uint16_t val);

  void SetSleepEnd(qhyccd_handle *handle,uint16_t val);

  void SetSleepFrames(qhyccd_handle *handle,uint16_t val);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  uint32_t SetChipWBRed(qhyccd_handle *h,double red);

  uint32_t SetChipWBGreen(qhyccd_handle *h,double green);

  uint32_t SetChipWBBlue(qhyccd_handle *h,double blue);

  uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);


  uint32_t SetChipOffset(qhyccd_handle *h,double offset);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);

  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);


  uint32_t SetChipBinMode(qhyccd_handle *h, uint32_t wbin, uint32_t hbin) ;

  static void ThreadCountExposureTime(void *p);

  void ThreadCountExposureTimeStart(qhyccd_handle *h);

  uint32_t BeginSingleExposure(qhyccd_handle *h);

  uint32_t GetSingleFrame(qhyccd_handle *h, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);

  uint32_t BeginLiveExposure(qhyccd_handle *h);

  uint32_t StopLiveExposure(qhyccd_handle *h);

  /** @fn uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY)
    @brief Set the camera on focus mode
    @param h camera control handle
    @param focusCenterX
    @param focusCenterY
    @return
    on success,return QHYCCD_SUCCESS \n
    another QHYCCD_ERROR code on other failures
   */
  uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY);

  void Sensor14bit_V_1_3(qhyccd_handle *h);

  void Sensor10bit_V_1_3_h_3bin(qhyccd_handle *h);

  void ResetParameters();

  void UpdateParameters(qhyccd_handle *h);

  uint32_t GeDDRBufferThreshold();
private:
#if defined (_WIN32)

  HANDLE hCountExpTimeThread;
#endif

  double pllratio,oldpllratio; //!< inter pll ratio

  uint32_t hmax_ref,vmax_ref;
  uint32_t vmax,hmax,oldvmax,oldhmax;
  uint32_t framelength,oldframelength;
  uint32_t oldchipoutputsizex,oldchipoutputsizey,oldchipoutputbits;
  uint32_t patchvnumber,oldpatchvnumber;
  int32_t shr,svr,spl,oldshr,oldsvr,oldspl;
  double oldcamgain;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;

  uint8_t oldreg08,reg08,oldreg01,reg01;
  uint8_t oldlockframe,lockframe;

  uint8_t oldcamviewmode;

  uint8_t oldcamampv;

  uint16_t oldcamddr;

  uint8_t forceupdate;

  uint8_t locknum,oldlocknum;

  double oldcamred2green,oldcamgreen,oldcamblue2green;

  uint32_t sleepframes,oldsleepframes,initsingleframe;

  uint32_t cutleftsize,cutrightsize,cuttopsize,cutbottomsize;


  int ApproxDDR;
  int ApproxDDR_Target;

};
#endif
