
#include "qhy5iiiddrcoolbase.h"
#include "config.h"
#ifndef QHY5III163BASE_DEF
#define QHY5III163BASE_DEF

#define DDR_BUFFER_IMAGE_THRESHOLD 32206
#define IMAGE_LENGTH_THRESHOLD (33030144 + 2048 * 20)

#define ACTIVE_PIXEL_WIDTH_DIMESION_MM 17.6472
#define ACTIVE_PIXEL_HEIGHT_DIMENSION_MM 13.3228
#define PIXEL_SIZE_UM 3.8
#define INIT_EXPOSURE_TIME 20000.0
#define CAM_OFFSET 64.0
#define TOTAL_PIXELS_WIDTH  4720
#define TOTAL_PIXELS_HEIGHT  3522
#define USB_ENDPOINT  0x81
#define CAM_BITS  16
#define USB_PACKET_LENGTH  4096
#define USB_TIMEOUT  1000

/**
 * @brief QHY163M class define
 *
 * include all functions for qhy163m
 */
class QHY5III163BASE : public QHY5IIIDDRCOOLBASE
{
public:
  QHY5III163BASE();
  virtual ~QHY5III163BASE();

  void writeCMOS(qhyccd_handle *pDevHandle, int addr, int value);
  void ThreadCountExposureTimeStart(qhyccd_handle *pDevHandle);

  //unsigned int readDDRNum(qhyccd_handle *pDevHandle);
  static void ThreadCountExposureTime(void *p);

  uint32_t GetUsedDdr(qhyccd_handle *pDevHandle, uint32_t *pDdr);
  uint32_t SetChipGain(qhyccd_handle *pDevHandle, double gain);
  uint32_t DisConnectCamera(qhyccd_handle *pDevHandle);
  uint32_t BeginSingleExposure(qhyccd_handle *pDevHandle);
  uint32_t GetSingleFrame(qhyccd_handle *pDevHandle, uint32_t *pW, uint32_t *pH, uint32_t *pBpp, uint32_t *pChannels, uint8_t *pImgData);
  uint32_t CancelExposingAndReadout(qhyccd_handle *pDevHandle);
  uint32_t CancelExposing(qhyccd_handle *pDevHandle);
  uint32_t BeginLiveExposure(qhyccd_handle *pDevHandle);
  uint32_t StopLiveExposure(qhyccd_handle *pDevHandle);
  uint32_t SetChipExposeTime(qhyccd_handle *pDevHandle, double i);
  uint32_t SetChipWBRed(qhyccd_handle *pDevHandle, double red);
  uint32_t SetChipWBBlue(qhyccd_handle *pDevHandle, double blue);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId, double *min, double *max, double *step);


  uint32_t SetChipOffset(qhyccd_handle *pDevHandle, double offset);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *pDevHandle,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *pDevHandle, uint32_t bits);

  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *pDevHandle, uint32_t x, uint32_t y, uint32_t xsize, uint32_t ysize);

  /** @fn uint32_t SetFocusSetting(qhyccd_handle *pDevHandle,uint32_t focusCenterX, uint32_t focusCenterY)
    @brief Set the camera on focus mode
    @param h camera control handle
    @param focusCenterX
    @param focusCenterY
    @return
    on success,return QHYCCD_SUCCESS \n
    another QHYCCD_ERROR code on other failures
   */
  uint32_t SetFocusSetting(qhyccd_handle *pDevHandle, uint32_t focusCenterX, uint32_t focusCenterY);
  uint32_t SetChipUSBTraffic(qhyccd_handle *pDevHandle,uint32_t i);
  uint32_t GeDDRBufferThreshold();

  void initcmos(qhyccd_handle *pDevHandle);

  void UpdateParameters(qhyccd_handle *pDevHandle);

  void ResetParameters();

private:
  //    static constexpr double ACTIVE_PIXEL_WIDTH_DIMESION_MM = 17.6472;
  //    static constexpr const double ACTIVE_PIXEL_HEIGHT_DIMENSION_MM = 13.3228;
  //    static constexpr const double PIXEL_SIZE_UM = 3.8;
  //    static constexpr const double INIT_EXPOSURE_TIME = 20000.0;
  //    static constexpr const double CAM_OFFSET = 64.0;
  //    static const int TOTAL_PIXELS_WIDTH = 4720;
  //    static const int TOTAL_PIXELS_HEIGHT = 3522;
  //    static const uint8_t USB_ENDPOINT = 0x81;
  //    static const uint32_t CAM_BITS = 16;
  //    static const uint32_t USB_PACKET_LENGTH = 4096;
  //    static const uint32_t USB_TIMEOUT = 1000;

  uint8_t m_UsbPacket[USB_PACKET_LENGTH];
  int32_t m_UsbTransferred;

  uint16_t digital_gain, analog_gain;

  double pllratio, oldpllratio; //!< inter pll ratio

  uint16_t vwinpos, oldvwinpos, vwidcut, oldvwidcut;

  uint8_t oldcamddr;

  uint32_t hmax_ref, vmax_ref;
  uint32_t vmax, hmax, oldvmax, oldhmax;
  uint32_t framelength, oldframelength;
  uint32_t oldchipoutputsizex, oldchipoutputsizey, oldchipoutputbits;
  uint32_t oldchipoutputx, oldchipoutputy;

  uint32_t patchvnumber, oldpatchvnumber;

  int32_t oldeleshutter, eleshutter;

  double oldtotalcamred2green, totalcamred2green, oldtotalcamgreen, totalcamgreen, oldtotalcamblue2green, totalcamblue2green, oldcamgain;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;

  uint8_t oldreg08, reg08, oldreg01, reg01;
  uint8_t oldlockframe, lockframe;

  uint16_t chiproiy, oldchiproiy, chiproiysize, oldchiproiysize;

  uint32_t oldampv_min, ampv_min, oldampv_max, ampv_max;
  uint8_t oldampv_onff, ampv_onff;
  uint32_t cutleftsize, cutrightsize, cuttopsize, cutbottomsize;


  int imageY;
  unsigned int ApproxDDR;
  unsigned int ApproxDDR_Target;

};
#endif
