
#include "qhy5iiicoolbase.h"
#include "config.h"
#ifndef QHY5III168BASE_DEF
#define QHY5III168BASE_DEF

//#define DDR_BUFFER_IMAGE_THRESHOLD 32206
//#define IMAGE_LENGTH_THRESHOLD (33030144 + 2048 * 20)

//#define ACTIVE_PIXEL_WIDTH_DIMESION_MM 17.6472
//#define ACTIVE_PIXEL_HEIGHT_DIMENSION_MM 13.3228
//#define PIXEL_SIZE_UM 3.8
//#define INIT_EXPOSURE_TIME 20000.0
//#define CAM_OFFSET 64.0
//#define TOTAL_PIXELS_WIDTH  4720
//#define TOTAL_PIXELS_HEIGHT  3522
#define USB_ENDPOINT  0x81
#define CAM_BITS  16
#define USB_PACKET_LENGTH  4096
#define USB_TIMEOUT  1000

/**
 * @brief QHY168C class define
 *
 * include all functions for qhy168c
 */
class QHY5III168BASE:public QHY5IIICOOLBASE
{
public:
  QHY5III168BASE();
  ~QHY5III168BASE();

  //unsigned int readDDRNum(qhyccd_handle *h);

  uint32_t GetUsedDdr(qhyccd_handle *pDevHandle, uint32_t *pDdr);
  uint32_t InitChipRegs(qhyccd_handle *pDevHandle);
  void SetFreqDiv(qhyccd_handle *pDevHandle,uint8_t val);
  void IMX071_TakePhoto(qhyccd_handle *pDevHandle);
  void WriteCMOSOffset(qhyccd_handle *pDevHandle,uint32_t val);
  void WriteCMOSAnalogGainRed(qhyccd_handle *pDevHandle,uint32_t val);
  void WriteCMOSAnalogGainBlue(qhyccd_handle *pDevHandle,uint32_t val);
  void WriteCMOSAnalogGainGreen(qhyccd_handle *pDevHandle,uint32_t val);
  void InitSPISleep(qhyccd_handle *pDevHandle);
  void FX3SPIMode(qhyccd_handle *pDevHandle);
  void FPGASPIMode(qhyccd_handle *pDevHandle);
  void WriteCMOS(qhyccd_handle *pDevHandle,uint16_t index,uint16_t val);
  void WriteCMOSSHS(qhyccd_handle *pDevHandle,uint32_t val);
  void SetCMOSCrop(qhyccd_handle *pDevHandle,uint32_t ystart,uint32_t ysize);
  uint32_t SetDDR(qhyccd_handle *pDevHandle, double value);
  void CalcFreqDiv(qhyccd_handle *pDevHandle);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *pDevHandle,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *pDevHandle,double gain);

  uint32_t SetChipWBRed(qhyccd_handle *pDevHandle,double red);

  uint32_t SetChipWBGreen(qhyccd_handle *pDevHandle,double green);

  uint32_t SetChipWBBlue(qhyccd_handle *pDevHandle,double blue);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *pDevHandle,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *pDevHandle,double i);
  void SetChipExposeTime_Internal(qhyccd_handle *pDevHandle,double i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId, double *min,double *max,double *step);


  uint32_t SetChipOffset(qhyccd_handle *pDevHandle,double offset);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *pDevHandle,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *pDevHandle,uint32_t bits);

  uint32_t SetChipUSBTraffic(qhyccd_handle *pDevHandle,uint32_t i);

  uint32_t SetChipBinMode(qhyccd_handle *h, uint32_t wbin, uint32_t hbin);
  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *pDevHandle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */

  uint32_t SetChipResolution(qhyccd_handle *pDevHandle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);

  static void ThreadCountExposureTime(void *p);

  void ThreadCountExposureTimeStart(qhyccd_handle *pDevHandle);

  uint32_t BeginSingleExposure(qhyccd_handle *pDevHandle);

  uint32_t GetSingleFrame(qhyccd_handle *pDevHandle, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);

  uint32_t BeginLiveExposure(qhyccd_handle *pDevHandle);

  uint32_t StopLiveExposure(qhyccd_handle *pDevHandle);

  void UpdateParameters(qhyccd_handle *pDevHandle);

  void ResetParameters();

  uint32_t GeDDRBufferThreshold() ;

private:
  uint8_t m_UsbPacket[USB_PACKET_LENGTH];
  int32_t m_UsbTransferred;
  uint8_t oldlockframe,lockframe;
  uint8_t digitalgain,olddigitalgain;
  uint8_t oldcamviewmode;

  uint32_t hmax_ref,vmax_ref;
  uint32_t vmax,hmax,oldvmax,oldhmax;
  uint32_t vmax_valid,oldvmax_valid,hmax_valid,oldhmax_valid;
  uint32_t framelength,oldframelength;
  uint32_t oldchipoutputsizex,oldchipoutputsizey,oldchipoutputbits;
  uint32_t patchvnumber,oldpatchvnumber;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;
  uint32_t sleepframes,oldsleepframes,sleepmode,initsingleframe;
  uint32_t cutleftsize,cutrightsize1,cutrightsize2,cuttopsize1,cuttopsize2,cuttopsize3,cutbottomsize,cutbuttomsize;

  int32_t shr,svr,spl,oldshr,oldsvr,oldspl;
  uint16_t oldcamddr;
  int ApproxDDR;
  int ApproxDDR_Target;
  int flag_after_roi_change;

  double oldcamblue2green,oldcamred2green,oldcamgreen,oldcamgain;
  double pllratio,oldpllratio; //!< inter pll ratio
};
#endif
