

#include "qhy5iiicoolbase.h"
#include "config.h"
#ifndef QHY5III183BASE_DEF
#define QHY5III183BASE_DEF


/**
 * @brief QHY5III183BASE class define
 *
 * include all functions for qhy5iii183base
 */
class QHY5III183BASE:public QHY5IIICOOLBASE
{
public:
  QHY5III183BASE();
  ~QHY5III183BASE();



  void WriteCMOS(qhyccd_handle *handle,uint16_t index,uint16_t val);
  //unsigned int readDDRNum(qhyccd_handle *h);

  uint32_t InitChipRegs(qhyccd_handle *h);

  uint32_t DisConnectCamera(qhyccd_handle *h);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);


  uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i);

  uint32_t SetChipOffset(qhyccd_handle *h,double offset);

  static void ThreadCountExposureTime(void *p);

  void ThreadCountExposureTimeStart(qhyccd_handle *h);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);

  uint32_t BeginSingleExposure(qhyccd_handle *h);
  uint32_t GetSingleFrame(qhyccd_handle *h, uint32_t *pW, uint32_t *pH, uint32_t * pBpp, uint32_t *pChannels, uint8_t *ImgData);


  uint32_t CancelExposing(qhyccd_handle *h);

  uint32_t CancelExposingAndReadout(qhyccd_handle *h);

  uint32_t BeginLiveExposure(qhyccd_handle *h);

  uint32_t StopLiveExposure(qhyccd_handle *h);

  uint32_t GetLiveFrame(qhyccd_handle *h,uint32_t *pW,uint32_t *pH,uint32_t * pBpp,uint32_t *pChannels,uint8_t *ImgData);
  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);

  /** @fn uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY)
    @brief Set the camera on focus mode
    @param h camera control handle
    @param focusCenterX
    @param focusCenterY
    @return
    on success,return QHYCCD_SUCCESS \n
    another QHYCCD_ERROR code on other failures
   */
  uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY);


  uint32_t SetStreamMode(qhyccd_handle *handle,uint8_t mode);


  void UpdateParameters(qhyccd_handle *h);

  void ResetParameters();
private:

  double pllratio,oldpllratio; //!< inter pll ratio

  uint16_t vwinpos,oldvwinpos,vwidcut,oldvwidcut;

  uint8_t oldcamddr;

  uint32_t hmax_ref,vmax_ref;
  uint32_t vmax,hmax,oldvmax,oldhmax;
  uint32_t framelength,oldframelength;
  uint32_t oldchipoutputsizex,oldchipoutputsizey,oldchipoutputbits;
  uint32_t patchvnumber,oldpatchvnumber;
  int32_t shr,svr,spl,oldshr,oldsvr,oldspl;
  double oldtotalcamred2green,totalcamred2green,oldtotalcamgreen,totalcamgreen,oldtotalcamblue2green,totalcamblue2green,oldcamgain;
  uint32_t oldcamoffset;
  uint32_t oldusbtraffic;

  uint8_t oldreg08,reg08,oldreg01,reg01;
  uint8_t oldlockframe,lockframe;

  uint16_t chiproiy,oldchiproiy,chiproiysize,oldchiproiysize;

  uint32_t oldampv_min,ampv_min,oldampv_max,ampv_max;
  uint8_t oldampv_onff,ampv_onff;

  uint8_t forcestop;

  int ApproxDDR,ApproxDDR_Target;
};
#endif
