

#include "qhy5iiibase.h"
#include "config.h"
#ifndef QHY5IIICOMMONBASE_DEF
#define QHY5IIICOMMONBASE_DEF


/**
 * @brief QHY367M class define
 *
 * include all functions for qhy367m
 */
class QHY5IIICOMMON:public QHY5IIIBASE
{
public:
  QHY5IIICOMMON();
  ~QHY5IIICOMMON();

  uint32_t InitChipRegs(qhyccd_handle *h);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  uint32_t SetChipWBRed(qhyccd_handle *h,double red);

  uint32_t SetChipWBGreen(qhyccd_handle *h,double green);

  uint32_t SetChipWBBlue(qhyccd_handle *h,double blue);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);


  uint32_t SetChipOffset(qhyccd_handle *h,double offset);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);


  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);


  /** @fn uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY)
    @brief Set the camera on focus mode
    @param h camera control handle
    @param focusCenterX
    @param focusCenterY
    @return
    on success,return QHYCCD_SUCCESS \n
    another QHYCCD_ERROR code on other failures
   */
  uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY);

  uint32_t GetChipMemoryLength();

  uint32_t IsChipHasFunction(CONTROL_ID controlId);

  uint32_t BeginLiveExposure(qhyccd_handle *h);

  uint32_t StopLiveExposure(qhyccd_handle *h);


private:

};
#endif
