

#include "qhy5iibase.h"
#include "config.h"

#ifndef QHY5RII_M_DEF
#define QHY5RII_M_DEF


/**
 * @brief QHY5RII_M class define
 *
 * include all functions for miniCam5s-m
 */
class QHY5RII_M:public QHY5IIBASE
{
public:
  QHY5RII_M();
  ~QHY5RII_M();

  /**
   @fn void InitCmos(qhyccd_handle *h)
   @brief Init the registers
   @param h camera control handle
   */
  void InitCmos(qhyccd_handle *h);

  /**
   @fn uint32_t IsChipHasFunction(CONTROL_ID id)
   @brief check the camera has the function or not
   @param id function id
   @return
   HAVE return QHYCCD_HAVE \n
   NOT HAVE return QHYCCD_NOTHAVE
   */
  uint32_t IsChipHasFunction(CONTROL_ID id);

  /**
   @fn uint32_t SetChipGain(qhyccd_handle *h,double gain)
   @brief set the gain to camera
   @param h camera control handle
   @param gain gain value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipGain(qhyccd_handle *h,double gain);

  uint32_t SetChipOffset(qhyccd_handle *h, double offset);

  uint32_t SetChipWBRed(qhyccd_handle *h,double red);

  uint32_t SetChipWBGreen(qhyccd_handle *h,double green);

  uint32_t SetChipWBBlue(qhyccd_handle *h,double blue);

  /**
   @fn uint32_t SetChipExposeTime(qhyccd_handle *h,double i)
   @brief set the expose time to camera
   @param h camera control handle
   @param i expose time value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipExposeTime(qhyccd_handle *h,double i);

  uint32_t BeginSingleExposure(qhyccd_handle *h);

  /**
   @fn uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i)
   @brief set the transfer speed to camera
   @param h camera control handle
   @param i speed level
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipSpeed(qhyccd_handle *h,uint32_t i);

  /**
   @fn uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step)
   @brief get the min,max and step value for function
   @param controlId the control id
   @param min the min value for function
   @param max the max value for function
   @param step single step value for function
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t GetControlMinMaxStepValue(CONTROL_ID controlId,double *min,double *max,double *step);

  /**
   @fn uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits)
   @brief set the camera depth bits
   @param h camera control handle
   @param bits depth bits
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipBitsMode(qhyccd_handle *h,uint32_t bits);

  /**
   @fn uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize)
   @brief set camera ouput resolution
   @param handle camera control handle
   @param x the top left position x
   @param y the top left position y
   @param xsize the image width
   @param ysize the image height
   @return
   on success,return QHYCCD_SUCCESS\n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipResolution(qhyccd_handle *handle,uint32_t x,uint32_t y,uint32_t xsize,uint32_t ysize);

  /**
   @fn uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i)
   @brief set hblank
   @param h camera control handle
   @param i hblank value
   @return
   success return QHYCCD_SUCCESS \n
   another QHYCCD_ERROR code on other failures
   */
  uint32_t SetChipUSBTraffic(qhyccd_handle *h,uint32_t i);

  /**
      @fn virtual uint32_t GetChipMemoryLength()
      @brief get the image cost memory length
      @return
      success return memory length \n
      another QHYCCD_ERROR code on other failures
      */
  uint32_t GetChipMemoryLength();

  /** @fn uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY)
    @brief Set the camera on focus mode
    @param h camera control handle
    @param focusCenterX
    @param focusCenterY
    @return
  on success,return QHYCCD_SUCCESS \n
  another QHYCCD_ERROR code on other failures
   */
  uint32_t SetFocusSetting(qhyccd_handle *h,uint32_t focusCenterX, uint32_t focusCenterY);

private:
  //uint32_t longtimeflag;
  uint32_t reg300c;
  uint32_t rgbgainb,rgbgainc;
  uint32_t rgbgainbluea,rgbgainreda;
};
#endif
